
/*
 * UNFS3 attribute handling
 * (C) 2004, Pascal Schmidt <der.eremit@email.de>
 * see file LICENSE for license details
 */

#include <ownfsd.h>
  
/* This is really stupid */
/* NFS can't help itself, it want inodes */
/* we don't DO inodes */
/* So we'll just return a unique number */
#ifdef OW_MT
    static pthread_mutex_t fileid_mutex = PTHREAD_MUTEX_INITIALIZER ;
    #define FILEIDLOCK       pthread_mutex_lock(  &fileid_mutex   )
    #define FILEIDUNLOCK     pthread_mutex_unlock(&fileid_mutex   )
#else /* OW_MT */
    #define FILEIDLOCK
    #define FILEIDUNLOCK
#endif /* OW_MT */

static fileid3 fileid = 0 ;


/*
 * find stat bit corresponding to given NFS file type
 */
mode_t type_to_mode(ftype3 ftype)
{
    switch (ftype) {
	case NF3REG:
	    return S_IFREG;
	case NF3DIR:
	    return S_IFDIR;
	case NF3LNK:
	    return S_IFLNK;
	case NF3CHR:
	    return S_IFCHR;
	case NF3BLK:
	    return S_IFBLK;
	case NF3FIFO:
	    return S_IFIFO;
	case NF3SOCK:
	    return S_IFSOCK;
    }

    /* fix gcc warning */
    return 0;
}

/*
 * post_op_attr for error returns
 */
static inline post_op_attr error_attr(void)
{
    post_op_attr result;

    result.attributes_follow = FALSE;
    return result;
}

void bad_pre_attr(pre_op_attr * attr) {
    /* default answers */
    attr->attributes_follow = FALSE;
}
            

/*
 * return pre-operation attributes
 *
 * fh_decomp must be called before to fill the stat cache
 */
void get_pre_attr(pre_op_attr * attr, const struct parsedname * pn) {
    /* default answers */
    attr->attributes_follow = TRUE;
    attr->pre_op_attr_u.attributes.size = 1;
    attr->pre_op_attr_u.attributes.mtime.seconds = start_time;
    attr->pre_op_attr_u.attributes.mtime.nseconds = 0;
    attr->pre_op_attr_u.attributes.ctime.seconds = start_time;
    attr->pre_op_attr_u.attributes.ctime.nseconds = 0;

    switch(pn->desc) {
    case desc_error:
        attr->attributes_follow = FALSE;
        break ;
    case desc_branchdir:
    case desc_device:
    case desc_alarmdir:
    case desc_simuldir:
    case desc_nondevice:
    case desc_subdir:
        FSTATLOCK;
            attr->pre_op_attr_u.attributes.mtime.seconds = dir_time;
            attr->pre_op_attr_u.attributes.ctime.seconds = dir_time;
        FSTATUNLOCK;
        break ;
    case desc_file:
        attr->pre_op_attr_u.attributes.size = FS_size_postparse(pn) ;
        switch (pn->ft->change) {
        case ft_volatile:
        case ft_Avolatile:
        case ft_second:
        case ft_statistic:
            attr->pre_op_attr_u.attributes.mtime.seconds = time(NULL);
            attr->pre_op_attr_u.attributes.ctime.seconds = time(NULL);
            break ;
        case ft_stable:
        case ft_Astable:
            FSTATLOCK;
                attr->pre_op_attr_u.attributes.mtime.seconds = dir_time;
                attr->pre_op_attr_u.attributes.ctime.seconds = dir_time;
            FSTATUNLOCK;
            break ;
        default:
            break ;
        }
    case desc_rootdir:
        break ;
    }
}

fileid3 UniqueNum( void ) {
    fileid3 f ;
    FILEIDLOCK;
        f = ++fileid ;
    FILEIDUNLOCK;
    return f ;
}

void bad_post_attr(post_op_attr * attr) {
    attr->attributes_follow = FALSE;
}

/*
 * compute post-operation attributes given a stat buffer
 */
void get_post_attr(post_op_attr * attr, const struct parsedname * pn) {
    struct stat stbuf ; /* for fstat call */

    if ( FS_fstat_low( &stbuf, pn ) ) {
        bad_post_attr(attr) ;
    } else {
        attr->attributes_follow = TRUE;
        
        attr->post_op_attr_u.attributes.size =
        attr->post_op_attr_u.attributes.used =
                stbuf.st_size ;
        
        attr->post_op_attr_u.attributes.mtime.nseconds =
        attr->post_op_attr_u.attributes.ctime.nseconds =
        attr->post_op_attr_u.attributes.atime.nseconds =
                0;
        
        attr->post_op_attr_u.attributes.mtime.seconds =
                stbuf.st_mtime ;
        attr->post_op_attr_u.attributes.ctime.seconds =
                stbuf.st_ctime ;
        attr->post_op_attr_u.attributes.atime.seconds =
                stbuf.st_atime ;
        attr->post_op_attr_u.attributes.mode = stbuf.st_mode & 0xFFFF;
        attr->post_op_attr_u.attributes.nlink = stbuf.st_nlink;

        attr->post_op_attr_u.attributes.uid = 0 ;
        attr->post_op_attr_u.attributes.gid = 0;

        attr->post_op_attr_u.attributes.rdev.specdata1 = 0;
        attr->post_op_attr_u.attributes.rdev.specdata2 =  0;
        attr->post_op_attr_u.attributes.fsid = 0;
        attr->post_op_attr_u.attributes.fileid = UniqueNum() ;

        switch(pn->desc) {
        case desc_nondevice:
        case desc_error:
            attr->attributes_follow = FALSE;
            break ;
        case desc_rootdir:
        case desc_branchdir:
        case desc_device:
        case desc_alarmdir:
        case desc_simuldir:
        case desc_subdir:
            attr->post_op_attr_u.attributes.type = NF3DIR;
            break ;
        case desc_file:
            attr->post_op_attr_u.attributes.type = NF3REG;
        }
    }
}

