/* OWNFSD rework */
/* Paul Alfille an the OWFS team */
/* owfs.sourceforge.net */

/*
 * UNFS3 low-level filehandle routines
 * (C) 2004, Pascal Schmidt <der.eremit@email.de>
 * see file LICENSE for license details
 */

#ifndef OWNFSD_H
#define OWNFSD_H

#include <owfs_config.h>
#include <ow.h>
#include <rpc/rpc.h>
#include <rpc/xdr.h>
#include "nfs.h"
#include "mount.h"
#include "xdr.h"

typedef struct {
    unsigned char      sn[8] ;    /* Serial number */
    struct device *    dev ;    /* pointer to dev tree */
    struct filetype *  ft ;    /* pointer filetype for this device type */
    int16_t            extension ;    /* extension */
    int16_t            type ;        /* pn_type */
    int16_t            state ;        /* pn_state */
    int16_t            pathlength ;    /* number of branches */
    uint16_t           pathbuffer ;    /* index into buffer of extra-long branching paths */
    uint32_t           verify ;    /* number to compare with pathbufferentry to ensure hasn't changed */
    int16_t            bus_nr ;        /* bus_nr */
    struct buspath     bp[2] ;    /* First 2 branches */
} __attribute__((packed)) unfs3_fh_t;

/* maximum depth of pathname described by filehandle */
#define FH_MAXLEN (sizeof(struct unfs3_fh_t ) ) ;
/* minimum length of complete filehandle */
#define FH_MINLEN FH_MAXLEN

void nfs3_program_3(struct svc_req *rqstp, register SVCXPRT * transp) ;
void mountprog_3(struct svc_req *rqstp, register SVCXPRT * transp) ;
void ow_exit( int e ) ;

struct datapasser_t {
    struct parsedname * pn ;
    char * result ;
    char * buffer ;
} ;

int fh_create(char * data_val, const struct parsedname * pn ) ;
int fh_interpret(const unfs3_fh_t * fh, struct parsedname * pn ) ;

fileid3 UniqueNum( void ) ;

void bad_pre_attr(pre_op_attr * attr) ;
void get_pre_attr(pre_op_attr * attr, const struct parsedname * pn) ;

void bad_post_attr(post_op_attr * attr) ;
void get_post_attr(post_op_attr * attr, const struct parsedname * pn) ;

/* daemon.h stuff */

#define NFSPROC3_NULL 0
#define NFSPROC3_GETATTR 1
void nfsproc3_getattr_3_svc(     GETATTR3args     * argp, struct datapasser_t * dp ) ;
#define NFSPROC3_SETATTR 2
void nfsproc3_setattr_3_svc(     SETATTR3args     * argp, struct datapasser_t * dp ) ;
#define NFSPROC3_LOOKUP 3
void nfsproc3_lookup_3_svc(      LOOKUP3args      * argp, struct datapasser_t * dp ) ;
#define NFSPROC3_ACCESS 4
void nfsproc3_access_3_svc(      ACCESS3args      * argp, struct datapasser_t * dp ) ;
#define NFSPROC3_READLINK 5
void nfsproc3_readlink_3_svc(    READLINK3args    * argp, struct datapasser_t * dp ) ;
#define NFSPROC3_READ 6
void nfsproc3_read_3_svc(        READ3args        * argp, struct datapasser_t * dp ) ;
#define NFSPROC3_WRITE 7
void nfsproc3_write_3_svc(       WRITE3args       * argp, struct datapasser_t * dp ) ;
#define NFSPROC3_CREATE 8
void nfsproc3_create_3_svc(      CREATE3args      * argp, struct datapasser_t * dp ) ;
#define NFSPROC3_MKDIR 9
void nfsproc3_mkdir_3_svc(       MKDIR3args       * argp, struct datapasser_t * dp ) ;
#define NFSPROC3_SYMLINK 10
void nfsproc3_symlink_3_svc(     SYMLINK3args     * argp, struct datapasser_t * dp ) ;
#define NFSPROC3_MKNOD 11
void nfsproc3_mknod_3_svc(       MKNOD3args       * argp, struct datapasser_t * dp ) ;
#define NFSPROC3_REMOVE 12
void nfsproc3_remove_3_svc(      REMOVE3args      * argp, struct datapasser_t * dp ) ;
#define NFSPROC3_RMDIR 13
void nfsproc3_rmdir_3_svc(       RMDIR3args       * argp, struct datapasser_t * dp ) ;
#define NFSPROC3_RENAME 14
void nfsproc3_rename_3_svc(      RENAME3args      * argp, struct datapasser_t * dp ) ;
#define NFSPROC3_LINK 15
void nfsproc3_link_3_svc(        LINK3args        * argp, struct datapasser_t * dp ) ;
#define NFSPROC3_READDIR 16
void nfsproc3_readdir_3_svc(     READDIR3args     * argp, struct datapasser_t * dp ) ;
#define NFSPROC3_READDIRPLUS 17
void nfsproc3_readdirplus_3_svc( READDIRPLUS3args * argp, struct datapasser_t * dp ) ;
#define NFSPROC3_FSSTAT 18
void nfsproc3_fsstat_3_svc(      FSSTAT3args      * argp, struct datapasser_t * dp ) ;
#define NFSPROC3_FSINFO 19
void nfsproc3_fsinfo_3_svc(      FSINFO3args      * argp, struct datapasser_t * dp ) ;
#define NFSPROC3_PATHCONF 20
void nfsproc3_pathconf_3_svc(    PATHCONF3args    * argp, struct datapasser_t * dp ) ;
#define NFSPROC3_COMMIT 21
void nfsproc3_commit_3_svc(      COMMIT3args      * argp, struct datapasser_t * dp ) ;

#endif /* OWNFSD */
